//
//  RootViewController.m
//  AdBasic
//
//  Created by Tomonari Shimada on 00/00/00.
//  Copyright 2011 Blueair Inc. All rights reserved.
//

#import "RootViewController.h"

// ネットワークの判定クラスのヘッダファイル読み込み
#import "Reachability.h"

@interface RootViewController (private) 
- (BOOL)checkInternet;
- (void)createAdBannerView;
- (void)showAdBanner;
- (void)hideAdBanner;
- (void)updateAdBanner;
- (void)changeAdBannerOrientation:(UIInterfaceOrientation)toInterfaceOrientation;
@end

@implementation RootViewController

@synthesize _tableView;
@synthesize internetReach;
@synthesize adBannerView;

#pragma mark -
#pragma mark Initialization

// ビューがロードされたら呼ばれる
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // タイトル
    self.title = @"AdBasic";
    
    // テーブルに表示するリスト
    itemList = [[NSArray alloc] initWithObjects:@"Apple", @"Google", @"Yahoo!", @"Microsoft", @"Facebook", @"アップル", @"グーグル", @"ヤフー", @"マイクロソフト", @"フェイスブック", nil];
    
    // ネットワークの状態を取得するクラスをインスタンス化
    internetReach = [[Reachability reachabilityForInternetConnection] retain];
    // ネットワークの状態を取得するメソッドをスタートさせる
	[internetReach startNotifier];
    
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっていて、まだバナーが作成されていなかったらバナーを作成するメソッドを呼ぶ
    if (reachable && (self.adBannerView == nil)) {
        [self createAdBannerView];
    }
    
    // ネットワークの状態が変化したら呼ばれるメソッドを設定
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:)
                                                 name:kReachabilityChangedNotification 
                                               object:nil];
    
}

// ビューが表示される前に呼ばれる
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    // コンテンツを更新するメソッドを呼び出す
    if (self.adBannerView && adBannerFlag) 
    {
		[self updateAdBanner];
	}
}

#pragma mark -
#pragma mark Device orientation

// デバイスの回転に対応するか設定
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

// デバイスが回転を始めたときに実行されるメソッド
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    // デバイスの向きに応じてバナーのサイズを変更するメソッドを呼び出す
    if (self.adBannerView) {
		[self changeAdBannerOrientation:toInterfaceOrientation];
	}
}

// デバイスが回転した後この処理が呼ばれるメソッド
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation 
{
    // デバイスの向きに応じてバナーとコンテンツを更新するメソッドを呼び出す
    if (self.adBannerView && adBannerFlag) {
		[self updateAdBanner];
	}
}

#pragma mark -
#pragma mark UITableViewDelegate methods

// セクションの数
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

// 項目の数
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [itemList count];
}

// セルの表示
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // テーブルのセルにリストの項目を表示
    cell.textLabel.text = [itemList objectAtIndex:indexPath.row];
    
    return cell;
}

// セルが選択されたときに呼ばれる
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
}

#pragma mark -
#pragma mark Memory management

// メモリ警告のとき呼ばれる
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// ビューがアンロードしたら呼ばれる
- (void)viewDidUnload
{
    adBannerView.delegate = nil;
    self.adBannerView = nil;
}

// メモリ解放処理
- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [internetReach release], internetReach = nil;
    adBannerView.delegate = nil;
    self.adBannerView = nil;
    [adBannerView release];
    [_tableView release], _tableView = nil;
    [itemList release], itemList = nil;
    [super dealloc];
}

#pragma mark -
#pragma mark Network check

// ネットワークに接続出来るか確認するメソッド
- (BOOL)checkInternet
{
    //
    NetworkStatus netStatus = [internetReach currentReachabilityStatus];
    BOOL connectionRequired = [internetReach connectionRequired];
    
    // ネットワークに繋がっているのであれば「YES」
    if ((netStatus == ReachableViaWiFi) || ((netStatus == ReachableViaWWAN) && !connectionRequired)) {
        return YES;
    }
    
    // ネットワークに繋がっていないのであれば「NO」
    return NO;
}

// ネットワークに変化があった際に呼ばれる
- (void)reachabilityChanged:(NSNotification *)note
{
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっていて、まだバナーが作成されていなかったらバナーを作成するメソッドを呼ぶ
    if (reachable && (self.adBannerView == nil)) {
        [self createAdBannerView];
    }
    
    // ネットワークに繋がっていない状態で、バナーが表示されていたら非表示にする
    if (!reachable && self.adBannerView) {
        [self hideAdBanner];
        adBannerView.delegate = nil;
        self.adBannerView = nil;
    }
}

#pragma mark -
#pragma mark iAd Banner View

NSString *adBannerContentSizeIdentifierPortrait;
NSString *adBannerContentSizeIdentifierLandscape;

// バナーを作成
- (void)createAdBannerView
{
    Class class = NSClassFromString(@"ADBannerView");
    if (class) {
        
        if (&ADBannerContentSizeIdentifierPortrait != nil) {
            adBannerContentSizeIdentifierPortrait = ADBannerContentSizeIdentifierPortrait;
        } else {
            adBannerContentSizeIdentifierPortrait = ADBannerContentSizeIdentifier320x50;
        }
		
        if (&ADBannerContentSizeIdentifierLandscape != nil) {
            adBannerContentSizeIdentifierLandscape = ADBannerContentSizeIdentifierLandscape;
        } else {
            adBannerContentSizeIdentifierLandscape = ADBannerContentSizeIdentifier480x32;
        }
		
        // デバイスの向きで表示するバナーのサイズを選択
        NSString *adContentSize = UIInterfaceOrientationIsPortrait(self.interfaceOrientation) ? 
        adBannerContentSizeIdentifierPortrait : adBannerContentSizeIdentifierLandscape;
        
        // バナーをインスタンス化
        ADBannerView *adView = [[ADBannerView alloc] initWithFrame:CGRectZero];
        adView.delegate = self;
        
        // バナーのサイズを設定
        adView.requiredContentSizeIdentifiers = [NSSet setWithObjects:adBannerContentSizeIdentifierPortrait, adBannerContentSizeIdentifierLandscape, nil];
        
        // 表示する広告のサイズを設定
        adView.currentContentSizeIdentifier = adContentSize;
        adView.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleRightMargin;
        
        //メインビューのサイズ
        CGRect viewFrame = self.view.bounds;
        
        // バナーの表示座標
        CGRect bannerFrame = CGRectMake(0, 
                                        viewFrame.size.height, 
                                        adView.frame.size.width, 
                                        adView.frame.size.height);
        adView.frame = bannerFrame;
        self.adBannerView = adView;
        
        // バナーをビューに表示する
        [self.view addSubview:adView];
        [adView release];
        
        adBannerFlag = FALSE;
    }
}

// バナーを表示
- (void)showAdBanner
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height - self.adBannerView.frame.size.height, 
                                    self.adBannerView.frame.size.width, 
                                    self.adBannerView.frame.size.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - self.adBannerView.frame.size.height);
	
    // バナーをアニメーションしながら表示
	[UIView beginAnimations:nil context:NULL];
	self.adBannerView.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = TRUE;
}

// バナーを非表示
- (void)hideAdBanner
{	
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height + self.adBannerView.frame.size.height, 
                                    self.adBannerView.frame.size.width, 
                                    self.adBannerView.frame.size.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height);
	
    // バナーをアニメーションしながら非表示
	[UIView beginAnimations:nil context:NULL];
	self.adBannerView.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = FALSE;
}

- (void)updateAdBanner 
{
    UIDeviceOrientation deviceOrientation = [UIDevice currentDevice].orientation;
    if (UIInterfaceOrientationIsLandscape(deviceOrientation)) {
		self.adBannerView.currentContentSizeIdentifier = adBannerContentSizeIdentifierLandscape;
	} else {
        self.adBannerView.currentContentSizeIdentifier = adBannerContentSizeIdentifierPortrait;
	}
    
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height - self.adBannerView.frame.size.height, 
                                    self.adBannerView.frame.size.width, 
                                    self.adBannerView.frame.size.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - self.adBannerView.frame.size.height);
    
    // バナーをアニメーションしながら表示
	[UIView beginAnimations:nil context:NULL];
	self.adBannerView.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
}

// デバイスの向きに応じてバナーのサイズを変更するメソッド
- (void)changeAdBannerOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    // デバイスが横のとき
	if (UIInterfaceOrientationIsLandscape(toInterfaceOrientation)) {
		self.adBannerView.currentContentSizeIdentifier = adBannerContentSizeIdentifierLandscape;
        // デバイスが縦のとき
	} else {
        self.adBannerView.currentContentSizeIdentifier = adBannerContentSizeIdentifierPortrait;
	}
}

#pragma mark -
#pragma mark ADBannerViewDelegate methods

// 広告をロードしたとき呼ばれる
- (void)bannerViewDidLoadAd:(ADBannerView *)banner
{
    // バナーを表示する
	[self showAdBanner];
}

// 広告のロードに失敗したとき呼ばれる
- (void)bannerView:(ADBannerView *)banner didFailToReceiveAdWithError:(NSError *)error
{
    // バナーを非表示にする
	[self hideAdBanner];
    
    // iAdから広告が提供されなかった場合は別の広告ネットワークのバナーを表示する
}

// バナーがタッチされたとき呼ばれる
-(BOOL)bannerViewActionShouldBegin:(ADBannerView *)banner willLeaveApplication:(BOOL)willLeave
{
    NSLog(@"広告の表示・アクションが始まります");
    // 広告が表示される前にメインの処理や表示を一時停止する必要があります
    
    
    return YES;
}

// 広告の表示・アクションが終了したら呼ばれる
-(void)bannerViewActionDidFinish:(ADBannerView *)banner
{
    NSLog(@"広告の表示・アクションが終了しました");
}

@end
