//
//  RootViewController.m
//  AdSampleAdMaker
//
//  Created by Tomonari Shimada on 00/00/00.
//  Copyright 2011 Blueair Inc. All rights reserved.
//

#import "RootViewController.h"
#import "AdMakerView.h"

#define ADMAKER_VIEW_WIDTH 320 
#define ADMAKER_VIEW_HEIGHT 50 

@interface RootViewController (private) 
- (void)updateAdBanner;
@end

@implementation RootViewController

@synthesize _tableView;

#pragma mark -
#pragma mark 

#pragma mark -
#pragma mark Initialization

// ビューがロードされたら呼ばれる
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // タイトル
    self.title = @"AdSampleAdMaker";
    
    // テーブルに表示するリスト
    itemList = [[NSArray alloc] initWithObjects:@"Apple", @"Google", @"Yahoo!", @"Microsoft", @"Facebook", @"アップル", @"グーグル", @"ヤフー", @"マイクロソフト", @"フェイスブック", nil];
    
    // AdMakerのバナーをインスタンス化
    AdMaker = [[AdMakerView alloc] init];
	[AdMaker setAdMakerDelegate:self];
    
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height, 
                                    ADMAKER_VIEW_WIDTH, 
                                    ADMAKER_VIEW_HEIGHT);
    
	[AdMaker setFrame:bannerFrame];
	[self.view addSubview:AdMaker.view];
	[AdMaker start];
}

// ビューが表示される前に呼ばれる
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self updateAdBanner];
}

#pragma mark -
#pragma mark Device orientation

// デバイスの回転に対応するか設定
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

// デバイスが回転を始めたときに実行されるメソッド
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
   AdMaker.view.hidden = YES;
}

// デバイスが回転した後この処理が呼ばれるメソッド
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation 
{
    // デバイスの向きに応じてバナーとコンテンツを更新するメソッドを呼び出す
    if (adBannerFlag) {
		[self updateAdBanner];
	}
}

#pragma mark -
#pragma mark UITableViewDelegate methods

// セクションの数
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

// 項目の数
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [itemList count];
}

// セルの表示
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // テーブルのセルにリストの項目を表示
    cell.textLabel.text = [itemList objectAtIndex:indexPath.row];
    
    return cell;
}

// セルが選択されたときに呼ばれる
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
}

#pragma mark -
#pragma mark Memory management

// メモリ警告のとき呼ばれる
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// ビューがアンロードしたら呼ばれる
- (void)viewDidUnload
{

}

// メモリ解放処理
- (void)dealloc
{
    [_tableView release], _tableView = nil;
    [itemList release], itemList = nil;
    [AdMaker release], AdMaker = nil;
    [super dealloc];
}

#pragma mark -
#pragma mark AdMaker

- (UIViewController*)currentViewControllerForAdMakerView:(AdMakerView*)view {
	return self;
}

- (NSArray*)adKeyForAdMakerView:(AdMakerView*)view {
    return [NSArray arrayWithObjects:@"adUrl", @"siteID", @"zoneID", nil];
}

// バナーを表示
- (void)showAdBanner
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake((viewFrame.size.width - ADMAKER_VIEW_WIDTH) / 2, 
                                    viewFrame.size.height - ADMAKER_VIEW_HEIGHT, 
                                    ADMAKER_VIEW_WIDTH, 
                                    ADMAKER_VIEW_HEIGHT);

    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - ADMAKER_VIEW_HEIGHT);
	
    // バナーをアニメーションしながら表示
	[UIView beginAnimations:nil context:NULL];
	AdMaker.view.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = TRUE;
}

// バナーを非表示
- (void)hideAdBanner
{	
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height + ADMAKER_VIEW_HEIGHT, 
                                    ADMAKER_VIEW_WIDTH, 
                                    ADMAKER_VIEW_HEIGHT);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height);
	
    // バナーをアニメーションしながら非表示
	[UIView beginAnimations:nil context:NULL];
	AdMaker.view.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = FALSE;
}

- (void)updateAdBanner 
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake((viewFrame.size.width - ADMAKER_VIEW_WIDTH) / 2, 
                                    viewFrame.size.height - ADMAKER_VIEW_HEIGHT, 
                                    ADMAKER_VIEW_WIDTH, 
                                    ADMAKER_VIEW_HEIGHT);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - ADMAKER_VIEW_HEIGHT);
    
	AdMaker.view.frame = bannerFrame;
    _tableView.frame = tableFrame;
    
    AdMaker.view.hidden = NO;
}

#pragma mark -
#pragma mark AdMakerDelegate methods

//広告の取得に成功した際に呼ばれます。
- (void)didLoadAdMakerView:(AdMakerView*)view {
    [self showAdBanner];
}

//広告の取得に失敗した際に呼ばれます。
- (void)didFailedLoadAdMakerView:(AdMakerView*)view {
    [self hideAdBanner];
}

//広告のページが表示された際に呼ばれます。
- (void)willAppearAdMakerScreen:(AdMakerView*)view {
    NSLog(@"広告のページが表示されました");
}

//広告のページが非表示になった際に呼ばれます。
- (void)willDismissAdMakerScreen:(AdMakerView*)view {
    NSLog(@"広告のページが非表示になりました");
}

@end
