//
//  RootViewController.m
//  RSSReader
//
//  Created by Tomonari Shimada on 00/00/00.
//  Copyright 2011 Blueair Inc. All rights reserved.
//

#import "RootViewController.h"
#import "Parser.h"
#import "Items.h"
#import "WebViewController.h"
#import "ImageStore.h"
#import "Reachability.h"
#import "AdWhirlView.h"
#import "AdMakerView.h"

#define _MY_AD_WHIRL_APPLICATION_KEY @"ADWHIRL_KEY"

#define ADMAKER_VIEW_WIDTH 320 
#define ADMAKER_VIEW_HEIGHT 50 

@interface RootViewController (private) 
- (void)loadData;
- (void)notReachable;
- (BOOL)checkInternet;
- (void)createAdBanner;
- (void)showAdBanner;
- (void)hideAdBanner;
- (void)updateAdBanner;
@end

@implementation RootViewController

@synthesize _tableView;
@synthesize internetReach;
@synthesize adView;

#pragma mark -
#pragma mark ImageStore 

- (void)imageStoreDidGetNewImage:(ImageStore *)sender url:(NSString *)url
{
	[_tableView reloadData];
}

#pragma mark -
#pragma mark Data 

//RSSの記事を読み込んでテーブルビューを更新
- (void)loadData 
{    
    if (!imageStore) {
		imageStore = [ImageStore new];
		imageStore.delegate = self;
	}
    
    //読み込み済みの記事データをクリア
    [[Items sharedItems] clearItems];
    
    //読み込むRSSのアドレスを定義
    NSString *rssURLString = NSLocalizedString(@"RSS URL", nil);
    
    //データを読み込んでパース
    Parser *rssXMLParseObject = [[Parser alloc] init];
    [rssXMLParseObject parseXMLFileAtURL:[NSURL URLWithString:rssURLString]];
    [rssXMLParseObject release];
    
    //テーブルビューを更新
    [_tableView reloadData];
    
    //テーブルの先頭に移動
    if([[Items sharedItems] getItemsCount] > 0) {
        [_tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] atScrollPosition:UITableViewScrollPositionTop animated:NO];
    }
}

- (void)reLoadData 
{
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっている
    if (reachable) {
        //データを読み込むメソッドを呼び出し
        [self loadData];
    } else {
        [self notReachable];
    }
}

#pragma mark -
#pragma mark View lifecycle
//ビューが読み込まれた後（インスタンス化された後）に実行
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    imageStore = nil;
    
    //タイトル
    self.title = NSLocalizedString(@"Title", nil);
    
    //リロードボタンのターゲットとアクションを設定
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                               target:self 
                                                                               action:@selector(reLoadData)];
    self.navigationItem.rightBarButtonItem = addButton;
    [addButton release];
    
    // ネットワークの状態を取得するクラスをインスタンス化
    internetReach = [[Reachability reachabilityForInternetConnection] retain];
    // ネットワークの状態を取得するメソッドをスタートさせる
	[internetReach startNotifier];
    
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっている
    if (reachable) {
        //データを読み込むメソッドを呼び出し
        [self loadData];
        
        if (self.adView == nil) {
            [self createAdBanner];
        }
        
    } else {
        [self notReachable];
    }
    
    // ネットワークの状態が変化したら呼ばれるメソッドを設定
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(reachabilityChanged:)
                                                 name:kReachabilityChangedNotification 
                                               object:nil];
    
    
}

//Viewが表示される直前に実行
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    //ネットワークインジゲータを止める
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
    
    // 選択されているセルを解除する
    NSIndexPath *indexPath;
    indexPath = [_tableView indexPathForSelectedRow];
    if (indexPath) {
        [_tableView deselectRowAtIndexPath:indexPath animated:YES];
    }
    
    [_tableView reloadData];
    
    // バナーとコンテンツを更新するメソッドを呼び出す
    if (self.adView && adBannerFlag) 
    {
		[self updateAdBanner];
	}
}

#pragma mark -
#pragma mark Device orientation

// デバイスの回転に対応するか設定
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

// デバイスが回転を始めたときに実行されるメソッド
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{   
    [super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    self.adView.hidden = YES;
    [self.adView rotateToOrientation:toInterfaceOrientation];
}

// デバイスが回転した後この処理が呼ばれるメソッド
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation 
{
    // デバイスの向きに応じてバナーとコンテンツを更新するメソッドを呼び出す
    if (self.adView && adBannerFlag) {
		[self updateAdBanner];
	}
}

#pragma mark -
#pragma mark Table view data source

//セクションの数を定義
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

//セルの数を定義
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if([[Items sharedItems] getItemsCount] == 0) {
		return 0;
	}
    //記事の合計を返す
    return [[Items sharedItems] getItemsCount];
}

//セルに上表を表示
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [_tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // Configure the cell.
    //記事のタイトル
    cell.textLabel.text = [[Items sharedItems] getItem:indexPath.row].titleString;
    //記事の概要
    cell.detailTextLabel.text = [[Items sharedItems] getItem:indexPath.row].descriptionString;
    
    cell.imageView.image = [imageStore getImage:[NSString stringWithFormat:@"%@", [[Items sharedItems] getItem:indexPath.row].enclosureString]];
    
    return cell;
}

#pragma mark -
#pragma mark Table view delegate

//セルが選択された時の処理
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
    NSError *error;
    if (![[GANTracker sharedTracker] setCustomVariableAtIndex:1
                                                         name:@"Link"
                                                        value:[[Items sharedItems] getItem:indexPath.row].linkString
                                                    withError:&error]) {
    }
    
    if (![[GANTracker sharedTracker] trackEvent:@"List"
                                         action:@"Item"
                                          label:@"Clicked"
                                          value:indexPath.row
                                      withError:&error]) {
    }
    
    //選択されたセルの記事をブラウザで表示する
    WebViewController *webViewController = [[WebViewController alloc] initWithNibName:@"WebViewController" bundle:nil];
    webViewController.titleString = [[Items sharedItems] getItem:indexPath.row].titleString;
    webViewController.linkString = [[Items sharedItems] getItem:indexPath.row].linkString;
    [self.navigationController pushViewController:webViewController animated:YES];
    [webViewController release];
}

#pragma mark -
#pragma mark Memory management

// メモリ警告のとき呼ばれる
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// ビューがアンロードしたら呼ばれる
- (void)viewDidUnload
{
    [self set_tableView:nil];
    adView.delegate = nil;
    self.adView = nil;
}

// メモリ解放処理
- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [imageStore release];
    [internetReach release];
    self.adView.delegate = nil;
    self.adView = nil;
    [adView release];
    [AdMaker release], AdMaker = nil;
    [_tableView release];
    [super dealloc];
}

#pragma mark -
#pragma mark Network check

// ネットワークに接続出来るか確認するメソッド
- (BOOL)checkInternet
{
    NetworkStatus netStatus = [internetReach currentReachabilityStatus];
    BOOL connectionRequired = [internetReach connectionRequired];
    
    // ネットワークに繋がっているのであれば「YES」
    if ((netStatus == ReachableViaWiFi) || ((netStatus == ReachableViaWWAN) && !connectionRequired)) {
        return YES;
    }
    
    // ネットワークに繋がっていないのであれば「NO」
    return NO;
}

// ネットワークに変化があった際に呼ばれる
- (void)reachabilityChanged:(NSNotification *)note
{
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっていて、まだバナーが作成されていなかったらバナーを作成するメソッドを呼ぶ
    if (reachable && (self.adView == nil)) {
        [self createAdBanner];
    }
    
    // ネットワークに繋がっていない状態で、バナーが表示されていたら非表示にする
    if (!reachable && self.adView) {
        [self hideAdBanner];
    }
}

- (void)notReachable {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Attention", nil) 
                                                    message:NSLocalizedString(@"Network not connected", nil) 
                                                   delegate:nil 
                                          cancelButtonTitle:NSLocalizedString(@"OK", nil) 
                                          otherButtonTitles:nil];
    [alert show];
    [alert release];
}

#pragma mark -
#pragma mark AdWhirl

// バナーを作成
- (void)createAdBanner
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // AdWhirlをインスタンス化
    adView = [AdWhirlView requestAdWhirlViewWithDelegate:self];
    CGSize adSize = [adView actualAdSize];
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height, 
                                    adSize.width, 
                                    adSize.height);
    
    [adView setFrame:bannerFrame];
    [self.view addSubview:adView];
    
    // AdMakerのバナーをインスタンス化
    AdMaker = [[AdMakerView alloc] init];
    [AdMaker setAdMakerDelegate:self];
    
    adBannerFlag = FALSE;
}

// バナーを表示
- (void)showAdBanner
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーのサイズ
    CGSize adSize = [adView actualAdSize];
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake((viewFrame.size.width - adSize.width) / 2,  
                                    viewFrame.size.height - adSize.height, 
                                    adSize.width, 
                                    adSize.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - adSize.height);
	
    // バナーをアニメーションしながら表示
	[UIView beginAnimations:nil context:NULL];
	self.adView.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = TRUE;
}

// バナーを非表示
- (void)hideAdBanner
{	
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーのサイズ
    CGSize adSize = [adView actualAdSize];
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake(0, 
                                    viewFrame.size.height + adSize.height, 
                                    adSize.width, 
                                    adSize.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height);
	
    // バナーをアニメーションしながら非表示
	[UIView beginAnimations:nil context:NULL];
	self.adView.frame = bannerFrame;
    _tableView.frame = tableFrame;
	[UIView commitAnimations];
    
    adBannerFlag = FALSE;
}

- (void)updateAdBanner 
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーのサイズ
    CGSize adSize = [adView actualAdSize];
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake((viewFrame.size.width - adSize.width) / 2,  
                                    viewFrame.size.height - adSize.height, 
                                    adSize.width, 
                                    adSize.height);
    
    // テーブルビューのサイズ
    CGRect tableFrame = CGRectMake(0, 
                                   0, 
                                   viewFrame.size.width, 
                                   viewFrame.size.height - adSize.height);
    
	self.adView.frame = bannerFrame;
    _tableView.frame = tableFrame;
    
    self.adView.hidden = NO;
}

#pragma mark -
#pragma mark AdWhirlDelegate methods

- (NSString *)adWhirlApplicationKey {
    return _MY_AD_WHIRL_APPLICATION_KEY;
}

- (UIViewController *)viewControllerForPresentingModalView {
    return self.navigationController;
}

- (void)adWhirlDidReceiveAd:(AdWhirlView *)adWhirlView {
    [self showAdBanner];
}

- (void)adWhirlDidFailToReceiveAd:(AdWhirlView *)adWhirlView usingBackup:(BOOL)yesOrNo{
    [self hideAdBanner];
}

- (void)performEventAdMaker:(AdWhirlView *)adWhirlView 
{
    //メインビューのサイズ
    CGRect viewFrame = self.view.bounds;
    
    // バナーの表示座標
    CGRect bannerFrame = CGRectMake((viewFrame.size.width - ADMAKER_VIEW_WIDTH) / 2,  
                                    viewFrame.size.height - ADMAKER_VIEW_HEIGHT, 
                                    ADMAKER_VIEW_WIDTH, 
                                    ADMAKER_VIEW_HEIGHT);
    
    [AdMaker setFrame:kAdWhirlViewDefaultFrame];
    [adWhirlView replaceBannerViewWith:AdMaker.view];
    [adWhirlView setFrame:bannerFrame];
    [self updateAdBanner];
    [AdMaker start];
}

#pragma mark -
#pragma mark AdMaker

- (UIViewController*)currentViewControllerForAdMakerView:(AdMakerView*)view {
	return self;
}

- (NSArray*)adKeyForAdMakerView:(AdMakerView*)view {
    return [NSArray arrayWithObjects:@"adUrl", @"siteID", @"zoneID", nil];
}

#pragma mark -
#pragma mark AdMakerDelegate methods

//広告の取得に成功した際に呼ばれます。
- (void)didLoadAdMakerView:(AdMakerView*)view {
    
}

//広告の取得に失敗した際に呼ばれます。
- (void)didFailedLoadAdMakerView:(AdMakerView*)view {
    
}

//広告のページが表示された際に呼ばれます。
- (void)willAppearAdMakerScreen:(AdMakerView*)view {
    NSLog(@"広告のページが表示されました");
}

//広告のページが非表示になった際に呼ばれます。
- (void)willDismissAdMakerScreen:(AdMakerView*)view {
    NSLog(@"広告のページが非表示になりました");
}

@end
